--[=[
Adirelle's oUF layout
(c) 2009-2011 Adirelle (adirelle@tagada-team.net)
All rights reserved.
--]=]

local _G, moduleName, private = _G, ...
local oUF_Adirelle, assert = _G.oUF_Adirelle, _G.assert
local oUF = assert(oUF_Adirelle.oUF, "oUF is undefined in oUF_Adirelle")

if oUF_Adirelle.SingleStyle then return end

--<GLOBALS
local _G = _G
local CreateFrame = _G.CreateFrame
local floor = _G.floor
local GameFontNormal = _G.GameFontNormal
local gsub = _G.gsub
local huge = _G.math.huge
local InCombatLockdown = _G.InCombatLockdown
local ipairs = _G.ipairs
local pairs = _G.pairs
local select = _G.select
local setmetatable = _G.setmetatable
local tsort = _G.table.sort
local strmatch = _G.strmatch
local tinsert = _G.tinsert
local UnitAffectingCombat = _G.UnitAffectingCombat
local UnitAlternatePowerInfo = _G.UnitAlternatePowerInfo
local UnitAlternatePowerTextureInfo = _G.UnitAlternatePowerTextureInfo
local UnitAura = _G.UnitAura
local UnitCanAssist = _G.UnitCanAssist
local UnitCanAttack = _G.UnitCanAttack
local UnitClass = _G.UnitClass
local UnitFrame_OnEnter = _G.UnitFrame_OnEnter
local UnitFrame_OnLeave = _G.UnitFrame_OnLeave
local UnitIsConnected = _G.UnitIsConnected
local UnitIsDeadOrGhost = _G.UnitIsDeadOrGhost
local unpack = _G.unpack
--GLOBALS>
local mmin, mmax = _G.min, _G.max

local GAP, BORDER_WIDTH, TEXT_MARGIN = private.GAP, private.BORDER_WIDTH, private.TEXT_MARGIN
local FRAME_MARGIN, AURA_SIZE = private.FRAME_MARGIN, private.AURA_SIZE

local backdrop, glowBorderBackdrop = oUF_Adirelle.backdrop, oUF_Adirelle.glowBorderBackdrop

local borderBackdrop = { edgeFile = [[Interface\Addons\oUF_Adirelle\media\white16x16]], edgeSize = BORDER_WIDTH }

local SpawnTexture, SpawnText, SpawnStatusBar = private.SpawnTexture, private.SpawnText, private.SpawnStatusBar

local function UpdateHealBar(bar, unit, current, max, incoming)
	if UnitIsDeadOrGhost(unit) or not UnitIsConnected(unit) then
		current, incoming  = 0, 0
	end
	if bar.incoming ~= incoming or bar.current ~= current or bar.max ~= max then
		bar.incoming, bar.current, bar.max = incoming, current, max
		local health = bar:GetParent()
		if current and max and incoming and incoming > 0 and max > 0 and current < max then
			local width = health:GetWidth()
			bar:SetPoint("LEFT", width * current / max, 0)
			bar:SetWidth(width * mmin(incoming, max-current) / max)
			bar:Show()
		else
			bar:Hide()
		end
	end
end

local function IncomingHeal_PostUpdate(bar, event, unit, incoming)
	return UpdateHealBar(bar, unit, bar.current, bar.max, incoming or 0)
end

local function Health_PostUpdate(healthBar, unit, current, max)
	local bar = healthBar:GetParent().IncomingHeal
	return UpdateHealBar(bar, unit, current, max, bar.incoming)
end

local function Auras_PostCreateIcon(icons, button)
	local cd, count, overlay = button.cd, button.count, button.overlay
	button.icon:SetTexCoord(5/64, 59/64, 5/64, 59/64)
	count:SetParent(cd)
	count:SetAllPoints(button)
	count:SetJustifyH("RIGHT")
	count:SetJustifyV("BOTTOM")
	overlay:SetParent(cd)
	overlay:SetTexture([[Interface\AddOns\oUF_Adirelle\media\icon_border]])
	overlay:SetTexCoord(0, 1, 0, 1)
	cd.noCooldownCount = true
	cd:SetReverse(true)
	cd:SetDrawEdge(true)
end

local function Auras_PostUpdateIcon(icons, unit, icon, index, offset)
	if not select(5, UnitAura(unit, index, icon.filter)) then
		icon.overlay:Hide()
	end
end

local LibDispellable = oUF_Adirelle.GetLib("LibDispellable-1.0")

local function IsMine(unit)
	return unit == "player" or unit == "vehicle" or unit == "pet"
end

local IsEncounterDebuff = oUF_Adirelle.IsEncounterDebuff

local canSteal = select(2, UnitClass("player")) == "MAGE"

local function Buffs_CustomFilter(icons, unit, icon, name, rank, texture, count, dtype, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura)
	icon.expires = (expires ~= 0) and expires or huge
	local priority, bigger = 0, false
	if IsEncounterDebuff(spellID) then
		priority = 5
	end
	if UnitCanAttack("player", unit) then
		if (canSteal and isStealable) or LibDispellable:CanDispel(unit, true, dtype, spellID) then
			priority, bigger = 3, true
		end
	elseif UnitCanAssist("player", unit) then
		if IsMine(caster) then
			priority, bigger = priority + 1, true
		end
		if canApplyAura then
			priority = priority + 1
		end
		if shouldConsolidate then
			priority = priority - 1
		end
		if duration == 0 then
			priority = priority - 1
		end
	end
	icon.priority, icon.bigger = priority, bigger
	return true
end

local function Debuffs_CustomFilter(icons, unit, icon, name, rank, texture, count, dtype, duration, expires, caster, isStealable, shouldConsolidate, spellID, canApplyAura, isBossDebuff)
	icon.expires = (expires ~= 0) and expires or huge
	if isBossDebuff or IsEncounterDebuff(spellID) then
		icon.priority = 5
	elseif UnitCanAssist("player", unit) and LibDispellable:CanDispel(unit, false, dtype, spellID) then
		icon.priority = 2
	elseif IsMine(caster) then
		icon.priority = 1
	else
		icon.priority = 0
	end
	icon.bigger = icon.priority > 0
	return true
end

local Auras_SetPosition
do
	local function CompareIcons(a, b)
		if a.bigger and not b.bigger then
			return true
		elseif not a.bigger and b.bigger then
			return false
		elseif a.priority == b.priority then
			if a.expires == b.expires then
				return a:GetID() < b:GetID()
			else
				return a.expires < b.expires
			end
		else
			return a.priority > b.priority
		end
	end

	local tmp = {}
	function Auras_SetPosition(icons, numIcons)
		if not icons or numIcons == 0 then return end
		local spacing = icons.spacing or 1
		local width, height = icons:GetSize()
		local size = icons.size
		local anchor = icons.initialAnchor or "BOTTOMLEFT"
		local growthx = icons.growthx or 1
		local growthy = icons.growthy or 1

		-- Sort auras
		local num = #icons
		for i = 1, mmax(num, #tmp) do
			tmp[i] = icons[i]
		end
		tsort(tmp, CompareIcons)

		-- Icon sizes
		local bigSize = mmax(8, mmin(size * 3 / 2, width, height))
		size = mmax(8, mmin(size, width, height))

		local i = 1
		local x, y = 0, 0

		-- Layout large icons
		if icons.enlarge then
			local step = bigSize + spacing
			local maxx = mmax(1, width - bigSize)
			while i <= num and tmp[i].bigger and x <= maxx do
				local button = tmp[i]
				if button:IsShown() then
					button:SetSize(bigSize, bigSize)
					button:ClearAllPoints()
					button:SetPoint(anchor, icons, anchor, growthx * x, 0)
					x = x + step
				end
				i = i + 1
			end
		end

		-- Layout normal-sized icons
		local baseX, step = x, size + spacing
		local maxx, maxy = mmax(1, width - size), mmax(1, height - size)
		while i <= num and x <= maxx and y <= maxy do
			local button = tmp[i]
			if button:IsShown() then
				button:SetSize(size, size)
				button:ClearAllPoints()
				button:SetPoint(anchor, icons, anchor, growthx * x, growthy * y)
				x = x + step
				if x > maxx then
					y = y + step
					if y >= bigSize then
						x = 0
					else
						x = baseX
					end
				end
			end
			i = i + 1
		end

		-- Hide remaining icons
		while i <= num do
			tmp[i]:Hide()
			i = i + 1
		end
	end
end

local function Auras_ForceUpdate(self, event, unit)
	if unit and unit ~= self.unit then return end
	if self.Buffs then
		self.Buffs:ForceUpdate()
	end
	if self.Debuffs then
		self.Debuffs:ForceUpdate()
	end
end

local function Power_PostUpdate(power, unit, min, max)
	if power.disconnected or UnitIsDeadOrGhost(unit) then
		power:SetValue(0)
	end
end

local function AltPowerBar_PostUpdate(bar, min, cur, max)
	local unit = bar.__owner.unit
	if not unit then return end
	bar.Label:SetText(select(10, UnitAlternatePowerInfo(unit)))
	local _, powerRed, powerGreen, powerBlue = UnitAlternatePowerTextureInfo(unit, 2)
	local r, g, b = oUF.ColorGradient((cur-min)/(max-min), powerRed, powerGreen, powerBlue, 1, 0, 0)
	bar:SetStatusBarColor(r, g, b)
end

-- Additional auxiliary bars
local function LayoutAuxiliaryBars(self)
	local bars = self.AuxiliaryBars
	if not bars then return end
	local anchor, offset = self, 0
	if self.Buffs and self.Buffs.side == "BOTTOM" then
		offset = - self.Buffs:GetHeight()
	end
	for i, bar in ipairs(bars) do
		if bar:IsShown() then
			bar:SetPoint("TOP", anchor, "BOTTOM", 0, -FRAME_MARGIN+offset)
			anchor, offset = bar, 0
		end
	end
end

local function LayoutAuxiliaryBars_Hook(bar)
	 return LayoutAuxiliaryBars(bar.__mainFrame)
end

local function AddAuxiliaryBar(self, bar)
	if not self.AuxiliaryBars then
		self.AuxiliaryBars = {}
	end
	tinsert(self.AuxiliaryBars, bar)
	bar.__mainFrame = self
	bar:HookScript('OnShow', LayoutAuxiliaryBars_Hook)
	bar:HookScript('OnHide', LayoutAuxiliaryBars_Hook)
end

-- General bar layout
local function LayoutBars(self)
	local width, height = self:GetSize()
	if not width or not height or width == 0 or height == 0 then return end
	self.Border:SetWidth(width + 2 * BORDER_WIDTH)
	self.Border:SetHeight(height + 2 * BORDER_WIDTH)
	local portrait = self.Portrait
	if portrait then
		portrait:SetSize(height, height)
	end
	local power = self.Power
	if power then
		local totalPowerHeight = height * 0.45 - GAP
		local powerHeight = totalPowerHeight
		if self.SecondaryPowerBar and self.SecondaryPowerBar:IsShown() then
			powerHeight = (totalPowerHeight - GAP) / 2
		end
		power:SetHeight(powerHeight)
		height = height - totalPowerHeight - GAP
	end
	self.Health:SetHeight(height)
	if self.AuxiliaryBars then
		LayoutAuxiliaryBars(self)
	end
end

local function Element_Layout(element)
	return LayoutBars(element.__owner)
end

local function CastBar_Update(castbar)
	local self = castbar.__owner
	if castbar:IsVisible() then
		local height = castbar:GetHeight()
		if height then
			castbar.Icon:SetSize(height, height)
		end
		self.Power:Hide()
	else
		self.Power:Show()
	end
end

local function XRange_PostUpdate(xrange, event, unit, inRange)
	xrange.__owner:SetAlpha(inRange and 1 or oUF.colors.outOfRange[4])
end

local function ApplyAuraPosition(self, target, initialAnchor, anchorTo, growthx, growthy, dx, dy)
	self:Debug('ApplyAuraPosition', target, initialAnchor, anchorTo, growthx, growthy, dx, dy)
	target.initialAnchor = initialAnchor
	target.growthx = growthx
	target.growthy = growthy
	target:ClearAllPoints()
	target:SetPoint(initialAnchor, self, anchorTo, dx * FRAME_MARGIN, dy * FRAME_MARGIN)
end

local function UpdateAuraCount(target, size, spacing)
	local width, height = target:GetSize()
	local step = mmax(8, mmin(size, width, height)) + spacing
	target.num = floor((width + spacing) / step) * floor((height + spacing) / step)
	target:ForceUpdate()
end

local function OnAuraLayoutModified(self, event, layout)
	local width, height = self:GetSize()
	local buffs, debuffs = self.Buffs, self.Debuffs

	local auras = layout.Single.Auras
	local size, spacing, side = auras.size, auras.spacing, auras.sides[self.baseUnit]
	buffs.size, buffs.spacing, buffs.enlarge, buffs.side = size, spacing, auras.enlarge, side
	if debuffs then
		debuffs.size, debuffs.spacing, debuffs.enlarge, debuffs.side = size, spacing, auras.enlarge, side
	end

	-- Apply position
	if side == 'LEFT' or side == 'RIGHT' then
		-- Left or right
		local dx, opposite
		if side == 'LEFT' then
			dx, opposite = -1, 'RIGHT'
		else
			dx, opposite = 1, 'LEFT'
		end
		ApplyAuraPosition(self, buffs, 'BOTTOM'..opposite, 'BOTTOM'..side, dx, -1, dx, 0)

		-- Ensure we can display at least 12 normal icons or 5 large ones
		local auraWidth = size * 12 + spacing * 11
		if auras.enlarge then
			auraWidth = mmax(mmin(size * 1.5, height) * 6 + spacing * 5)
		end

		-- Share the available space with debuffs, if necessary
		if debuffs then
			ApplyAuraPosition(self, debuffs, 'TOP'..opposite, 'TOP'..side, dx, 1, dx, 0)
			buffs:SetSize(auraWidth, height / 2)
			debuffs:SetSize(auraWidth, height / 2)
		else
			buffs:SetSize(auraWidth, height)
		end
	else
		-- Top or bottom
		local dy, opposite
		if side == 'TOP' then
			dy, opposite = 1, 'BOTTOM'
		else
			dy, opposite = -1, 'TOP'
		end

		-- Ensure we can display at least two rows of normal icons, or a row of large ones plus a row of normal ones
		local auraHeight = size * 2 + spacing
		if auras.enlarge then
			auraHeight = mmax(auraHeight, mmin(size * 1.5, width) + spacing + size)
		end

		ApplyAuraPosition(self, buffs, opposite..'LEFT', side..'LEFT', 1, dy, 0, dy)
		if debuffs then
			ApplyAuraPosition(self, debuffs, opposite..'RIGHT', side..'RIGHT', -1, dy, 0, dy)
			buffs:SetSize(width / 2, auraHeight)
			debuffs:SetSize(width / 2, auraHeight)
		else
			buffs:SetSize(width, auraHeight)
		end
	end

	-- Update the number of icons and update them
	UpdateAuraCount(buffs, size, spacing)
	if debuffs then
		UpdateAuraCount(debuffs, size, spacing)
	end

	-- Update auxiliary bars, just in case
	return LayoutAuxiliaryBars(self)
end

local function OnSingleLayoutModified(self, event, layout, theme)
	local width, height = layout.Single.width, layout.Single['height'..self.heightType]
	if self.heightFactor then
		height = height * self.heightFactor
	end
	if self:CanChangeProtectedState() and (self:GetWidth() ~= width or self:GetHeight() ~= height) then
		self:SetSize(width, height)
	end
end

local function OnSingleThemeModified(self, event, layout, theme)
	-- Update health coloring flags
	local health = self.Health
	for k,v in pairs(theme.Health) do
		health[k] = v
	end
	if self.baseUnit == "arena" then
		health.colorSmooth = false
	end
	health:ForceUpdate()

	-- Update power coloring flags
	local power = self.Power
	if power then
		for k,v in pairs(theme.Power) do
			power[k] = v
		end
		power:ForceUpdate()
	end
end

local function OnThemeModified(self, event, layout, theme)
	-- Update border settings
	local border = self.Border
	for k, v in pairs(theme.Border) do
		border[k] = v
	end
	border:ForceUpdate()

	-- Update low health threshold
	local lowHealth = self.LowHealth
	if lowHealth then
		local prefs = theme.LowHealth
		lowHealth.threshold = prefs.isPercent and -prefs.percent or prefs.amount
		lowHealth:ForceUpdate()
	end
end

local function OnColorModified(self, event, layout, theme)
	if self.LowHealth then
		self.LowHealth:SetTexture(unpack(oUF.colors.lowHealth, 1, 4))
	end
	if self.XRange then
		self.XRange:SetTexture(unpack(oUF.colors.outOfRange, 1, 3))
		self.XRange:ForceUpdate()
	end
	if self.IncomingHeal then
		self.IncomingHeal:SetTexture(unpack(oUF.colors.incomingHeal.self, 1, 4))
	end
	self.Health:ForceUpdate()
	if self.Power then
		self.Power:ForceUpdate()
	end
end

local DRAGON_TEXTURES = {
	rare  = { [[Interface\Addons\oUF_Adirelle\media\rare_graphic]],  6/128, 123/128, 17/128, 112/128, },
	elite = { [[Interface\Addons\oUF_Adirelle\media\elite_graphic]], 6/128, 123/128, 17/128, 112/128, },
}

local function OoC_UnitFrame_OnEnter(...)
	if not InCombatLockdown() then return UnitFrame_OnEnter(...) end
end

local function InitFrame(settings, self, unit)
	local unit = gsub(unit or self.unit, "%d+", "")
	local isArenaUnit = strmatch(unit, "arena")
	self.baseUnit, self.isArenaUnit = unit, isArenaUnit
	self.heightType = settings.heightType

	self:SetSize(settings['initial-width'], settings['initial-height'])
	if unit == "pet" then
		self.heightFactor = 40/47
	end

	self:RegisterForClicks("AnyDown")

	self:SetScript("OnEnter", OoC_UnitFrame_OnEnter)
	self:SetScript("OnLeave", UnitFrame_OnLeave)

	if self:CanChangeAttribute() then
		self:SetAttribute("type", "target")
	end

	private.SetupUnitDropdown(self, unit)

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0,0,0,backdrop.bgAlpha)
	self:SetBackdropBorderColor(0,0,0,0)

	-- Register setting callbacks early
	self:RegisterMessage('OnSingleLayoutModified', OnSingleLayoutModified)
	self:RegisterMessage('OnSettingsModified', OnSingleLayoutModified)
	self:RegisterMessage('OnSingleThemeModified', OnSingleThemeModified)
	self:RegisterMessage('OnSettingsModified', OnSingleThemeModified)
	self:RegisterMessage('OnColorModified', OnColorModified)
	self:RegisterMessage('OnSettingsModified', OnColorModified)
	self:RegisterMessage('OnSettingsModified', OnThemeModified)
	self:RegisterMessage('OnThemeModified', OnThemeModified)

	-- Border
	local border = CreateFrame("Frame", nil, self)
	border:SetFrameStrata("BACKGROUND")
	border:SetPoint("CENTER", self)
	border:SetBackdrop(borderBackdrop)
	border:SetBackdropColor(0, 0, 0, 0)
	border:SetBackdropBorderColor(1, 1, 1, 1)
	border.SetColor = border.SetBackdropBorderColor
	border.blackByDefault = true
	border.noTarget = true
	self.Border = border

	local barContainer
	local left, right, dir = "LEFT", "RIGHT", 1
	if settings.mirroredFrame then
		left, right, dir = "RIGHT", "LEFT", -1
	end

	-- Portrait
	if not settings.noPortrait then
		-- Spawn the player model
		local portrait = CreateFrame("PlayerModel", nil, self)
		portrait:SetPoint(left)
		self.Portrait = portrait

		-- Create an icon displaying important debuffs (either PvP or PvE) all over the portrait
		local importantDebuff = self:CreateIcon(portrait)
		importantDebuff:SetAllPoints(portrait)
		local stack = importantDebuff.Stack
		stack:ClearAllPoints()
		stack:SetPoint("BOTTOMRIGHT", importantDebuff, -1, 1)
		importantDebuff.Stack:SetFont(GameFontNormal:GetFont(), 14, "OUTLINE")
		self.WarningIcon = importantDebuff

		-- Spawn a container frame that spans remaining space
		barContainer = CreateFrame("Frame", nil, self)
		barContainer:SetPoint("TOP"..left, portrait, "TOP"..right, GAP*dir, 0)
		barContainer:SetPoint("BOTTOM"..right)

		-- Have the bars cover the whole frame if the portrait is disabled
		portrait:SetScript('OnShow', function() barContainer:SetPoint("TOP"..left, portrait, "TOP"..right, GAP*dir, 0) end)
		portrait:SetScript('OnHide', function() barContainer:SetPoint("TOP"..left, self, "TOP"..left, 0, 0) end)
	else
		barContainer = self
	end
	self.BarContainer = barContainer

	-- Health bar
	local health = SpawnStatusBar(self, false, "TOPLEFT", barContainer)
	health:SetPoint("TOPRIGHT", barContainer)
	health.frequentUpdates = true
	self.Health = health

	-- Name
	local name = SpawnText(health, "OVERLAY", "TOPLEFT", "TOPLEFT", TEXT_MARGIN)
	name:SetPoint("BOTTOMLEFT", health, "BOTTOMLEFT", TEXT_MARGIN)
	name:SetPoint("RIGHT", health.Text, "LEFT")
	self:Tag(name, (unit == "player" or unit == "pet" or unit == "boss" or isArenaUnit) and "[name]" or "[name][ <>status<>]")
	self.Name = name

	if unit ~= "boss" then
		-- Low health indicator
		local lowHealth = self:CreateTexture(nil, "OVERLAY")
		lowHealth:SetPoint("TOPLEFT", self, -2, 2)
		lowHealth:SetPoint("BOTTOMRIGHT", self, 2, -2)
		self.LowHealth = lowHealth

		-- Incoming heals
		local incomingHeal = health:CreateTexture(nil, "OVERLAY")
		incomingHeal:SetBlendMode("ADD")
		incomingHeal:SetPoint("TOP", health)
		incomingHeal:SetPoint("BOTTOM", health)
		incomingHeal.PostUpdate = IncomingHeal_PostUpdate
		incomingHeal.current, incomingHeal.max, incomingHeal.incoming = 0, 0, 0
		self.IncomingHeal = incomingHeal

		-- PostUpdate is only needed with incoming heals
		health.PostUpdate = Health_PostUpdate
	end

	-- Used for some overlays
	local indicators = CreateFrame("Frame", nil, self)
	indicators:SetAllPoints(self)
	indicators:SetFrameLevel(health:GetFrameLevel()+3)
	self.Indicators = indicators

	-- Power bar
	if not settings.noPower then
		local power = SpawnStatusBar(self, false, "TOPLEFT", health, "BOTTOMLEFT", 0, -GAP)
		power:SetPoint('RIGHT', barContainer)
		power.frequentUpdates = true
		power.PostUpdate = Power_PostUpdate
		self.Power = power

		if unit == "player" and private.SetupSecondaryPowerBar then
			-- Add player specific secondary power bar
			local bar = private.SetupSecondaryPowerBar(self)
			if bar then
				bar:SetPoint('TOPLEFT', self.Power, 'BOTTOMLEFT', 0, -GAP)
				bar:SetPoint('BOTTOMRIGHT', self.BarContainer)
				bar.__owner = self
				bar:HookScript('OnShow', Element_Layout)
				bar:HookScript('OnHide', Element_Layout)
				self.SecondaryPowerBar = bar
			end
		end

		-- Unit level and class (or creature family)
		if unit ~= "player" and unit ~= "pet" then
			local classif = SpawnText(power, "OVERLAY")
			classif:SetPoint("TOPLEFT", self.Health, "BOTTOMLEFT", 0, -GAP)
			classif:SetPoint("BOTTOM", barContainer)
			classif:SetPoint("RIGHT", power.Text, "LEFT")
			self:Tag(classif, "[smartlevel][ >smartclass<]")
		end

		-- Casting Bar
		if unit ~= 'player' then
			local castbar = CreateFrame("StatusBar", nil, self)
			castbar:Hide()
			castbar.__owner = self
			castbar:SetPoint('BOTTOMRIGHT', power)
			castbar.PostCastStart = function() castbar:SetStatusBarColor(1.0, 0.7, 0.0) end
			castbar.PostChannelStart = function() castbar:SetStatusBarColor(0.0, 1.0, 0.0) end
			castbar:SetScript('OnSizeChanged', CastBar_Update)
			castbar:SetScript('OnShow', CastBar_Update)
			castbar:SetScript('OnHide', CastBar_Update)
			self:RegisterStatusBarTexture(castbar)
			self.Castbar = castbar

			local icon = castbar:CreateTexture(nil, "ARTWORK")
			icon:SetPoint('TOPLEFT', power)
			icon:SetTexCoord(4/64, 60/64, 4/64, 60/64)
			castbar.Icon = icon

			local spellText = SpawnText(castbar, "OVERLAY")
			spellText:SetPoint('TOPLEFT', castbar, 'TOPLEFT', TEXT_MARGIN, 0)
			spellText:SetPoint('BOTTOMRIGHT', castbar, 'BOTTOMRIGHT', -TEXT_MARGIN, 0)
			castbar.Text = spellText

			castbar:SetPoint("TOPLEFT", icon, "TOPRIGHT", GAP, 0)
			CastBar_Update(castbar)
		end
	end

	-- Threat Bar
	if unit == "target" then
		-- Add a simple threat bar on the target
		local threatBar = SpawnStatusBar(self, false)
		threatBar:SetBackdrop(backdrop)
		threatBar:SetBackdropColor(0,0,0,backdrop.bgAlpha)
		threatBar:SetBackdropBorderColor(0,0,0,1)
		threatBar:SetWidth(190*0.5)
		threatBar:SetHeight(14)
		threatBar:SetMinMaxValues(0, 100)
		threatBar.PostUpdate = function(self, event, unit, bar, isTanking, status, scaledPercent, rawPercent, threatValue)
			if not bar.Text then return end
			if threatValue then
				local value, unit = threatValue / 100, ""
				if value > 1000000 then
					value, unit = value / 1000000, "m"
				elseif value > 1000 then
					value, unit = value / 1000, "k"
				end
				bar.Text:SetFormattedText("%d%% (%.1f%s)", scaledPercent, value, unit)
				bar.Text:Show()
			else
				bar.Text:Hide()
			end
		end
		self.ThreatBar = threatBar
		AddAuxiliaryBar(self, threatBar)
	end

	-- Raid target icon
	self.RaidIcon = SpawnTexture(indicators, 16)
	self.RaidIcon:SetPoint("CENTER", barContainer)

	-- Threat glow
	local threat = CreateFrame("Frame", nil, self)
	threat:SetAllPoints(self.Border)
	threat:SetBackdrop(glowBorderBackdrop)
	threat:SetBackdropColor(0,0,0,0)
	threat.SetVertexColor = threat.SetBackdropBorderColor
	threat:SetAlpha(glowBorderBackdrop.alpha)
	threat:SetFrameLevel(self:GetFrameLevel()+2)
	self.SmartThreat = threat

	if unit ~= "boss" and not isArenaUnit then
		-- Various indicators
		self.Leader = SpawnTexture(indicators, 16, "TOP"..left)
		self.Assistant = SpawnTexture(indicators, 16, "TOP"..left)
		self.MasterLooter = SpawnTexture(indicators, 16, "TOP"..left, 16*dir)
		self.Combat = SpawnTexture(indicators, 16, "BOTTOM"..left)

		-- Indicators around the portrait, if there is one
		if self.Portrait then
			-- Group role icons
			self.RoleIcon = SpawnTexture(indicators, 16)
			self.RoleIcon:SetPoint("CENTER", self.Portrait, "TOP"..right)
			self.RoleIcon.noRaidTarget = true

			-- PvP flag
			local pvp = SpawnTexture(indicators, 16)
			pvp:SetTexCoord(0, 0.6, 0, 0.6)
			pvp:SetPoint("CENTER", self.Portrait, "BOTTOM"..right)
			self.PvP = pvp

			-- PvP timer
			if unit == "player" then
				local timer = CreateFrame("Frame", nil, indicators)
				timer:SetAllPoints(pvp)
				timer.text = SpawnText(timer, "OVERLAY")
				timer.text:SetPoint("CENTER", pvp)
				self.PvPTimer = timer
			end
		end
	end

	if unit == "player" then
		-- Player resting status
		self.Resting = SpawnTexture(indicators, 16, "BOTTOMLEFT")

	elseif unit == "target" then
		-- Combo points
		local DOT_SIZE = 10
		local cpoints = {}
		for i = 0, 4 do
			local cpoint = SpawnTexture(indicators, DOT_SIZE)
			cpoint:SetTexture([[Interface\AddOns\oUF_Adirelle\media\combo]])
			cpoint:SetTexCoord(3/16, 13/16, 5/16, 15/16)
			cpoint:SetPoint("LEFT", health, "BOTTOMLEFT", i*(DOT_SIZE+GAP), 0)
			cpoint:Hide()
			tinsert(cpoints, cpoint)
		end
		self.ComboPoints = cpoints
	end

	-- Auras
	local buffs, debuffs
	if unit == "pet" then
		buffs = CreateFrame("Frame", nil, self)
		buffs:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, FRAME_MARGIN)
		buffs.initialAnchor = "BOTTOMLEFT"
		buffs.growthx = 1
		buffs.growthy = -1

	elseif unit == "target" or unit == "focus" or unit == "boss" or unit == "arena" then
		buffs = CreateFrame("Frame", nil, self)
		buffs:SetPoint("BOTTOM"..right, self, "BOTTOM"..left, -FRAME_MARGIN*dir, 0)
		buffs.showType = true
		buffs.initialAnchor = "BOTTOM"..right
		buffs.growthx = (left == "LEFT") and -1 or 1
		buffs.growthy = 1

		debuffs = CreateFrame("Frame", nil, self)
		debuffs:SetPoint("TOP"..right, self, "TOP"..left, -FRAME_MARGIN*dir, 0)
		debuffs.showType = true
		debuffs.initialAnchor = "TOP"..right
		debuffs.growthx = (left == "LEFT") and -1 or 1
		debuffs.growthy = -1
	end

	if buffs then
		buffs.size = AURA_SIZE
		buffs.num = 12
		buffs:SetSize(AURA_SIZE * 12, AURA_SIZE)
		buffs.CustomFilter = Buffs_CustomFilter
		buffs.SetPosition = Auras_SetPosition
		buffs.PostCreateIcon = Auras_PostCreateIcon
		buffs.PostUpdateIcon = Auras_PostUpdateIcon
		self.Buffs = buffs
	end
	if debuffs then
		debuffs.size = AURA_SIZE
		debuffs.num = 12
		debuffs:SetSize(AURA_SIZE * 12, AURA_SIZE)
		debuffs.CustomFilter = Debuffs_CustomFilter
		debuffs.SetPosition = Auras_SetPosition
		debuffs.PostCreateIcon = Auras_PostCreateIcon
		debuffs.PostUpdateIcon = Auras_PostUpdateIcon
		self.Debuffs = debuffs
	end

	if buffs or debuffs then
		self:RegisterEvent('UNIT_FACTION', Auras_ForceUpdate)
		self:RegisterEvent('UNIT_TARGETABLE_CHANGED', Auras_ForceUpdate)
		self:RegisterMessage('OnSettingsModified', OnAuraLayoutModified)
		self:RegisterMessage('OnSingleLayoutModified', OnAuraLayoutModified)
	end

	-- Classification dragon
	if unit == "target" or unit == "focus" or unit == "boss" then
		local dragon = indicators:CreateTexture(nil, "ARTWORK")
		local DRAGON_HEIGHT = 45*95/80+2
		dragon:SetWidth(DRAGON_HEIGHT*117/95)
		dragon:SetHeight(DRAGON_HEIGHT)
		dragon:SetPoint('TOPLEFT', self, 'TOPLEFT', -44*DRAGON_HEIGHT/95-1, 15*DRAGON_HEIGHT/95+1)
		dragon.elite = DRAGON_TEXTURES.elite
		dragon.rare = DRAGON_TEXTURES.rare
		self.Dragon = dragon
	end

	-- Experience Bar for player
	if unit == "player" then
		local xpFrame = CreateFrame("Frame", nil, self)
		xpFrame:SetPoint("TOP")
		xpFrame:SetPoint("RIGHT")
		xpFrame:SetHeight(12)
		xpFrame:SetBackdrop(backdrop)
		xpFrame:SetBackdropColor(0,0,0,backdrop.bgAlpha)
		xpFrame:SetBackdropBorderColor(0,0,0,1)
		xpFrame:EnableMouse(false)

		local xpBar = SpawnStatusBar(self, true)
		xpBar:SetParent(xpFrame)
		xpBar:SetAllPoints(xpFrame)
		xpBar.Show = function() return xpFrame:Show() end
		xpBar.Hide = function() return xpFrame:Hide() end
		xpBar.IsShown = function() return xpFrame:IsShown() end
		xpBar:EnableMouse(false)

		local restedBar = SpawnStatusBar(self, true)
		restedBar:SetParent(xpFrame)
		restedBar:SetAllPoints(xpFrame)
		restedBar:EnableMouse(false)

		local levelText = SpawnText(xpBar, "OVERLAY", "TOPLEFT", "TOPLEFT", TEXT_MARGIN, 0)
		levelText:SetPoint("BOTTOMLEFT", xpBar, "BOTTOMLEFT", TEXT_MARGIN, 0)

		local xpText = SpawnText(xpBar, "OVERLAY", "TOPRIGHT", "TOPRIGHT", -TEXT_MARGIN, 0)
		xpText:SetPoint("BOTTOMRIGHT", xpBar, "BOTTOMRIGHT", -TEXT_MARGIN, 0)

		local smartValue = private.smartValue
		xpBar.UpdateText = function(self, bar, current, max, rested, level)
			levelText:SetFormattedText(level)
			if rested and rested > 0 then
				xpText:SetFormattedText("%s(+%s)/%s", smartValue(current), smartValue(rested), smartValue(max))
			else
				xpText:SetFormattedText("%s/%s", smartValue(current), smartValue(max))
			end
		end

		xpBar.Rested = restedBar
		xpBar:SetFrameLevel(restedBar:GetFrameLevel()+1)

		self.ExperienceBar = xpBar
		AddAuxiliaryBar(self, xpFrame)
	end

	-- Range indicator
	if unit ~= "player" then
		local xrange = indicators:CreateTexture(nil, "BACKGROUND")
		xrange:SetAllPoints(self)
		xrange:SetTexture(0.4, 0.4, 0.4)
		xrange:SetBlendMode("MOD")
		xrange.PostUpdate = XRange_PostUpdate
		self.XRange = xrange
	end

	-- Special events
	if unit == "boss" then
		self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", self.UpdateAllElements)
	end
	self:RegisterEvent("UNIT_TARGETABLE_CHANGED", function(_, event, unit)
		if unit == self.unit then return self:UpdateAllElements(event)	end
	end)

	-- Altenate power bar (e.g. sound on Atramedes, or poison on Isorath)
	if unit == "player" or unit == "target" then

		local altPowerBar = SpawnStatusBar(self)
		altPowerBar:SetBackdrop(backdrop)
		altPowerBar:SetBackdropColor(0,0,0,backdrop.bgAlpha)
		altPowerBar:SetBackdropBorderColor(0,0,0,1)
		altPowerBar:SetPoint("LEFT")
		altPowerBar:SetPoint("RIGHT")
		altPowerBar:SetHeight(12)
		altPowerBar.showOthersAnyway = true
		altPowerBar.textureColor = { 1, 1, 1, 1 }
		altPowerBar.PostUpdate = AltPowerBar_PostUpdate

		local label = SpawnText(altPowerBar, "OVERLAY", "TOPLEFT", "TOPLEFT", TEXT_MARGIN, 0)
		label:SetPoint("RIGHT", altPowerBar.Text, "LEFT", -TEXT_MARGIN, 0)
		altPowerBar.Label = label

		self.AltPowerBar = altPowerBar
		AddAuxiliaryBar(self, altPowerBar)
	end

	-- Update layout at least once
	self:HookScript('OnSizeChanged', LayoutBars)
	LayoutBars(self)
end

local single_style = setmetatable({
	["initial-width"] = 190,
	["initial-height"] = 47,
	heightType = 'Big',
}, {
	__call = InitFrame,
})

oUF:RegisterStyle("Adirelle_Single", single_style)

local single_style_right = setmetatable({
	mirroredFrame = true
}, {
	__call = InitFrame,
	__index = single_style,
})

oUF:RegisterStyle("Adirelle_Single_Right", single_style_right)

local single_style_health = setmetatable({
	["initial-height"] = 20,
	heightType = 'Small',
	noPower = true,
	noPortrait = true
}, {
	__call = InitFrame,
	__index = single_style,
})

oUF:RegisterStyle("Adirelle_Single_Health", single_style_health)

oUF_Adirelle.SingleStyle = true
